@@ -20,15 +20,11 @@
 #ifndef FREERDP_CHANNEL_GEOMETRY_CLIENT_MAIN_H
 #define FREERDP_CHANNEL_GEOMETRY_CLIENT_MAIN_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <freerdp/config.h>
 
 #include <freerdp/dvc.h>
 #include <freerdp/types.h>
 #include <freerdp/addin.h>
 #include <freerdp/client/geometry.h>
 
-
 #endif /* FREERDP_CHANNEL_GEOMETRY_CLIENT_MAIN_H */
-
